# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2019
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          sh641Const.py
@date:          2019-12-21

@note           Definitions for Command/Response Strings
                  * ESPEC CORP. SH-641
"""


# Commands
CMD_GET_TYPE="TYPE?"             # request chamber identification
CMD_GET_TEMP="TEMP?"             # request chamber temperature
CMD_SET_TEMP="TEMP,S"            # chamber temperature set point
CMD_SET_TEMP_ALUP="TEMP,H"       # chamber alarm upper temperature
CMD_SET_TEMP_ALLOW="TEMP,L"      # chamber alarm lower temperature
CMD_SET_PWR="POWER,"             # chamber power on
PWR_ON="ON"                      # chamber switch on
PWR_OFF="OFF"                    # chamber switch off
CMD_SET_MODE="MODE,"             # chamber mode command
MODE_CONSTANT="CONSTANT"         # constant temperature mode
MODE_STANDBY="STANDBY"           # chamber in standby, no temperature set
MODE_OFF="OFF"                   # disables panel
CMD_GET_HUMI="HUMI?"             # request chamber humidity

# Responses
RSP_CH_ID="T,T,S2,160.0"         # ID of chamber
RSP_OK="OK"                      # Chamber accepts command
RSP_FAIL="NA"                    # chamber discards command

# Misc
MSC_LINE_END="\r\n"              # used line end
MSC_TIOUT_RS232_MSEC=10e3        # Time out for serial read
MSC_TEMP_RESOLUTION=0.1          # Resolution temperature chamber

# Interface
IF_DFLT_CFG="sh641InterfaceDefault.yml"

# Help Constant



# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2020
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          simChamber.py
@date:          2020-02-22

@note           virtual climate chamber, allows ATWG virtual run
"""


#------------------------------------------------------------------------------
# System Libs
#
import os         # platform independent paths 
#------------------------------------------------------------------------------



#------------------------------------------------------------------------------
class simChamber:

    #*****************************
    def __init__(self):
        self.last_set_temp = 20.0
    #*****************************
    
    
    #*****************************
    def open(self, cfgFile=None):
        """
        @note           opens physical interface to chamber, dummy in sim
        
        @rtype          boolean
        @return         successfull opened
        """
        # Config file is meaningless in sim mode
        if ( None != cfgFile ):
            raise Warning("Interface configurtion File '" + os.path.basename(cfgFile) + "' skipped in sim mode")
        # normal end
        return True
    #*****************************
    
    
    #*****************************
    def close(self):
        """
        @note           closes physical interface, dummy in sim
        
        @rtype          boolean
        @return         successfull closed
        """
        return True
    #*****************************
    
    
    #*****************************
    def start(self, temperature=None):
        """
        @note           starts chamber, dummy in sim
        
        @rtype          boolean
        @return         successfull closed
        """
        if ( None != temperature ):
            self.last_set_temp = temperature
        return True
    #*****************************
    
    
    #*****************************
    def stop(self):
        """
        @note           stops chamber, dummy in sim
        
        @rtype          boolean
        @return         successfull closed
        """
        return True
    #*****************************
    
    
    #*****************************
    def info(self):
        """
        @note           Get Humidity and Temperature fracs resolution
        
        @rtype          dict
        @return         hudidity/temperature resolution
        """
        # build fracs dict
        fracs = {}
        fracs['temperature'] = 2   # temperature frac digits
        fracs['humidity'] = 2      # humidity frac digits
        # build final info structure
        info = {}
        info['fracs'] = fracs   # add
        info['name'] = "SIM"    # insert
        # release
        return info
    #*****************************
    
    
    #*****************************
    def get_clima(self):
        """
        @note           Current measured clima
        
        @rtype          dict
        @return         hudidity/temperature vals
        """
        clima = {}
        clima['temperature'] = self.last_set_temp
        clima['humidity'] = float('nan')
        return clima
    #*****************************    
    
    
    #*****************************
    def set_clima(self, clima=None):
        """
        @note           Current measured clima
        
        @rtype          boolean
        @return         hudidity/temperature vals
        """
        # check for arg
        if ( clima == None ):
            raise ValueError("No new data provided")
        # try to set temperature
        try:
            self.last_set_temp = clima['temperature']
        except:
            raise ValueError("Miss temperature set value")
        # try to set humidity
        # graceful end
        return True
    #*****************************
    
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
if __name__ == '__main__':

    myChamber = simChamber()                        # call class constructor
    myChamber.open()                                # open with interface defaults
    print(myChamber.get_clima())                    # get current clima
    myChamber.start()                               # start chamber
    myChamber.set_clima(clima={'temperature': 25})  # set temperature value
    myChamber.stop()                                # stop chamber
    myChamber.close()                               # close chamber handle
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2019
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          sh641.py
@date:          2019-07-23

@note           Provides basic functions to interface the climate chamber
                    * ESPEC CORP. SH-641


Pinning:
 * COM-Port
 * Table lists chamber site
     |--------+--------+----------------------------------+------+
     | Number | Signal | Remark                           | Used |
     |--------+--------+----------------------------------+------+
     |    1   | --     | Protective ground / cable shield | no   |
     |    2   | TXD    | Transmited data                  | yes  |
     |    3   | RXD    | Received data                    | yes  |
     |    4   | DSR    | Data set ready                   | yes  |
     |    5   | GND    | Signal ground                    | yes  |
     |    6   | DTR    | Data terminal ready              | yes  |
     |    7   | CTS    | Clear to send                    | yes  |
     |    8   | RTS    | Request to send                  | yes  |
     |    9   | --     | Ground                           | no   |
     |--------+--------+----------------------------------+------+     
"""



#------------------------------------------------------------------------------
import os                  # platform independent paths
import serial              # COM port Interface
import math                # required for isnan
import yaml                # port config
from . import sh641Const   # ESPEC SH641 constants
#------------------------------------------------------------------------------



#------------------------------------------------------------------------------
class especShSu:

    #*****************************
    def __init__(self):
        """
        Initialization of class
        """
        # Com interface
        self.interface = None   # init variable with open
        self.sim = None
        self.sim_rd = ""        # stores answer of next read request
        # managment flags
        self.isOpen = False;    # interface is open
        # internal
        self.last_write_temp = float("nan") # stores last written value, used for reduction
    #*****************************


    #*****************************
    def open(self, cfgFile=None, simFile=None):
        """
        Opens COM port and try to recognize the climate chamber
        SRC: http://www.varesano.net/blog/fabio/serial%20rs232%20connections%20python
        """
        # Clima chamber interface mode
        if ( None == simFile ):
            # default interface config
            if ( None == cfgFile ):
                cfgFile = os.path.dirname(os.path.abspath(__file__)) + os.path.sep + sh641Const.IF_DFLT_CFG
            # check if file exists
            if ( False == os.path.isfile(cfgFile) ):
                raise ValueError("Interface configuration file '" + cfgFile + "' not found")
            # load interface config
            fH = open(cfgFile, 'r+')                                # open file for yaml loader
            self.interface = yaml.load(fH, Loader=yaml.FullLoader)  # load condig
            fH.close();                                             # close file handle
            # open interface
            #   https://pyserial.readthedocs.io/en/latest/
            self.com = serial.Serial(
                         port=self.interface['rs232'][os.name],     # port determined on os
                         baudrate=self.interface['baudrate'],       # current baudrate
                         stopbits=self.interface['stopbit'],
                         parity=self.interface['parity'],           # see 'serial.PARITY_NONE' for proper definition
                         bytesize=self.interface['databit'],
                         timeout=self.interface['tiout_sec']
                       )
        # simulation mode, Req/Res from file
        else:
            # User info
            print("Enter simulation mode with dialog file '" + os.path.basename(simFile) + "'")
            # exists?
            if ( False == os.path.isfile(simFile) ):
                raise ValueError("Dialog file '" + simFile + "' not found") 
            # open file and load
            fH = open(simFile, 'r+')                            # open file for yaml loader
            self.sim = yaml.load(fH, Loader=yaml.FullLoader)    # load condig
            fH.close();                                         # close file handle        
        # mark interface/sim as open
        self.isOpen = True
        # try to indentify chamber
        self.write(sh641Const.CMD_GET_TYPE)                 # request type
        chamberID = self.read()                             # read chamber repsonse
        if (False == (sh641Const.RSP_CH_ID in chamberID) ): # known type?
            raise ValueError("Error: Chamber '" + chamberID + "' unknown")
        # end
        return True
    #*****************************


    #*****************************
    def close(self):
        """
        Closes Handle
        """
        self.com.close()
        self.isOpen = False
    #*****************************


    #*****************************
    def write(self, msg):
        """
        @note           Writes buffer to serial port or prepares answer for next read in case of sim
        
        @param msg      command for chamber
        @rtype          boolean
        @return         if succesfull
        """
        # interface open or sim mode?
        if ( False == self.isOpen ):
            raise ValueError("Interface nor sim mode used") 
        # prepare record answer
        if ( None != self.sim ):
            # make empty
            self.sim_rd = ""
            # request command
            if ("?" == msg[-1]):
                self.sim_rd = self.sim['req'][msg[:-1]]     # add to next read buffer
            # set command, build ack message
            else:
                self.sim_rd = sh641Const.RSP_OK + ":" + msg
        # pyhsical interface used
        else:
            # bring to line 
            msg += sh641Const.MSC_LINE_END # append termination
            self.com.write(msg.encode())   # write to com
        # all fine
        return True
    #*****************************


    #*****************************
    def read(self):
        """
        @note           reads from serial port into buffer
        
        @rtype          string
        @return         chamber reponse in ASCII text
        """
        # interface open or sim mode?
        if ( False == self.isOpen ):
            raise ValueError("Interface nor sim mode used")         
        # make empty
        msg = ""
        # prepare record answer
        if ( None != self.sim ):
            msg = self.sim_rd   # respond to previous request
            self.sim_rd = ""    # clear
        # pyhsical interface used
        else:
            # read from COM
            while ( False == (sh641Const.MSC_LINE_END in msg) ):
                byte = self.com.read(1);
                msg += byte.decode()
            # drop line end and return
            msg = msg[:-len(sh641Const.MSC_LINE_END)]   # skip CRNL
            msg = msg.strip()                           # remove leading/trailing blanks
        # all done
        return msg
    #*****************************


    #*****************************
    def parse(self, msg):
        """
        @note           parses chamber responses
                          * set command
                          * get command
                          
        @param msg      chamber response string
        @type           string
        @rtype          dict
        @return         {'state': , 'parm': , 'val', :} in case of measuement
                        command contents contents val a dict
        """
        # check for message
        if ( 0 == len(msg) ):
            raise ValueError("Empty message to parse provided")
        # dict
        myParse = {}
        # Set Command / Failed Request
        if ( -1 != msg.find(":") ):
            # split at ':', f.e. OK:TEMP,S25
            msg = msg.split(':')        # OK:TEMP,S25 -> ['OK', 'TEMP,S25']
            myParse['state'] = msg[0]   # 'OK'
            msg = msg[1];               # 'TEMP,S25'
            # split at ',', f.e. 'TEMP,S25'
            msg = msg.split(',')        # 'TEMP,S25' -> ['TEMP', 'S25']
            # iterate over list
            i = 0                       # helps to align
            myParse['parm'] = ""
            myParse['val'] = ""         # for recursive alignment
            for elem in msg:
                if ( 0 == i ):  myParse['parm'] = elem
                if ( 0 < i ):   myParse['val'] = myParse['val'] + elem + ','    # collect rest of param
                i += 1
            # drop last ',' in 'val'
            myParse['val'] = myParse['val'][0:-1]
        # measurement command
        else:
            # prepare dicts
            myParse['state'] = sh641Const.RSP_OK    # 'OK'
            myParse['parm'] = "MEAS"                # measurement command was applied
            myMeas = {}                             # dict for measurment values
            myMeas['measured'] = float("nan")       # always full dict is returned
            myMeas['setpoint'] = float("nan")       #
            myMeas['upalarm']  = float("nan")       #
            myMeas['lowalarm'] = float("nan")       #
            # parse response
            i = 0
            for elem in msg.split(','):
                # non-numeric input, skip iteration
                if ( False == self.is_numeric(elem.strip()) ):
                    i+=1
                    continue
                # allign to dict
                if ( 0 == i ): myMeas['measured'] = float(elem.strip())                 # measured
                if ( 1 == i ): myMeas['setpoint'] = float(elem.strip())                 # set-point
                if ( 2 == i ): myMeas['upalarm'] = float(elem.strip())                  # upper-limit-alarm-value
                if ( 3 == i ): myMeas['lowalarm'] = float(elem.strip())                 # lower-limit-alarm-value                
                if ( 3 < i): raise ValueError("Unrecognized response in '" + msg + "'") # something went wrong
                i+=1
            # assign to reponse dict
            myParse['val'] = myMeas;
        # release parsed structure
        return myParse
    #*****************************
    
    
    #*****************************
    def is_numeric(self, msg):
        """
        @note           checks if input string is a numeric value
        
        @rtype          boolean
        @return         true if input is numeric
        """
        # clean message
        msg = msg.replace(' ', '')  # remove blanks
        msg = msg.replace('-', '')  # negative number
        msg = msg.replace('+', '')  # positive number
        msg = msg.replace('.', '')  # decimal point
        # check
        return msg.isnumeric()
    #*****************************   


    #*****************************
    def get_clima(self):
        """
        @note           Current measured clima
        
        @rtype          dict
        @return         humidity/temperature vals
        """
        # init resut
        clima = {'temperature': float('nan'), 'humidity': float('nan')}
        # acquire temperature
        try:
            self.write(sh641Const.CMD_GET_TEMP)             # write temperature request to chamber
            rsp = self.parse(self.read())                   # read/parse; dict: measured, setpoint, upalarm, lowalarm
            if not ( (sh641Const.RSP_OK == rsp['state']) and ("MEAS" == rsp['parm']) ):
                raise ValueError("Get temperaure request not succesfull completeted by chamber")
            clima['temperature'] = rsp['val']['measured']   # extract current temp values
        except:
            raise ValueError("Failed to get temperature not proper handled")
        # acquire humidity
        try:
            self.write(sh641Const.CMD_GET_HUMI)         #  write temperature request to chamber
            rsp = self.parse(self.read())               # read/parse; dict: measured, setpoint, upalarm, lowalarm
            if not ( (sh641Const.RSP_OK == rsp['state']) and ("MEAS" == rsp['parm']) ):
                raise ValueError("Get humidity request not succesfull completeted by chamber")
            clima['humidity'] = rsp['val']['measured']  # extract humidity values
        except:
            raise ValueError("Get humidity request not proper handled")        
        # release result
        return clima
    #*****************************


    #*****************************
    def set_clima(self, clima=None):
        """
        @note           set chambers new clima value
                          * temperature
                          
        @param clima    new clima value
        @type           dict, {'temperature': myVal}
        @rtype          boolean
        @return         successful
        """
        # check for arg
        if ( clima == None ):
            raise ValueError("No new data provided")
        # try to set temperature
        try:
            # check if update is necessary
            if ( False == math.isnan(self.last_write_temp) ):
                if ( sh641Const.MSC_TEMP_RESOLUTION >= abs(self.last_write_temp-clima['temperature']) ):
                    return True
            # prepare       
            numDigs = len(str(sh641Const.MSC_TEMP_RESOLUTION).split(".")[1])            # determine number of digits in fracs based on resulotion
            self.last_write_temp = clima['temperature']                                 # write only new value, if change is bigger then resulotion
            setTemp = '{temp:.{frac}f}'.format(temp=clima['temperature'], frac=numDigs) # build temp string based  on chambers fraction settings
            # request chamber
            try:
                self.write(sh641Const.CMD_SET_TEMP + setTemp)   # set new temperature
                rsp=self.parse(self.read())                     # read response from chamber
            except:
                raise ValueError("Request chamber failed")
            # check setting of new temperature
            #   rsp['val']: S35 -> 35
            if not ( (sh641Const.RSP_OK == rsp['state']) and ("TEMP" == rsp['parm']) and (float(setTemp) == float(rsp['val'][1:])) ):
                raise Warning("Temperature set check failed")
        except:
            raise ValueError("Failed to set clima")
        # graceful end
        return True
    #*****************************


    #*****************************
    def set_power(self, pwr=sh641Const.PWR_OFF):
        """
        Enables Disables Power of climate chamber

        Argument:
            on:
                False: disable chamber
                True:  enable chamber

        Return
            False: Something went wrong
            True:  Action successful performed
        """
        # check for proper arg
        if ( False == (pwr in (sh641Const.PWR_OFF, sh641Const.PWR_ON)) ):
            raise ValueError("unsupported power mode '" + pwr + "'")
        # request chamber
        try:
            self.write(sh641Const.CMD_SET_PWR + pwr)    # set power state
            rsp=self.parse(self.read())                 # read response from chamber
        except:
            raise ValueError("Request chamber failed")
        # check response
        if not ( (sh641Const.RSP_OK == rsp['state']) and ("POWER" == rsp['parm']) and (pwr == rsp['val']) ):
            raise ValueError("Failed to set new power state")
        # graceful end
        return True
    #*****************************


    #*****************************
    def set_mode(self, mode=sh641Const.MODE_STANDBY):
        """
        Selects Chamber mode

        Argument:
            mode:   ( MODE_CONSTANT MODE_STANDBY MODE_OFF )

        Return
            False: Something went wrong
            True:  Action succesfull performed
        """
        # check for proper arg
        if ( False == (mode in (sh641Const.MODE_CONSTANT, sh641Const.MODE_STANDBY, sh641Const.MODE_OFF)) ):
            raise ValueError("unsupported operating mode '" + mode + "'")
        # request chamber
        try:
            self.write(sh641Const.CMD_SET_MODE + mode)  # set mode
            rsp=self.parse(self.read())                 # read response from chamber
        except:
            raise ValueError("Request chamber failed")
        # check response
        if not ( (sh641Const.RSP_OK == rsp['state']) and ("MODE" == rsp['parm']) and (mode == rsp['val']) ):
            raise ValueError("Failed to set new mode")
        # graceful end
        return True        
    #*****************************


    #*****************************
    def start(self, temperature=None):
        """
        Starts temperature chamber

        Return
            False: Something went wrong
            True:  Successful
        """
        # check if start temp provided, othweise use chambers current temperature
        if ( None == temperature ):
            temperature = self.get_clima()['temperature']
        # start chamber
        try:
            self.set_clima(clima={'temperature': temperature})  # set start temp
            self.set_power(sh641Const.PWR_ON)                   # enable chamber
            self.set_mode(sh641Const.MODE_CONSTANT)             # run in constant mode
        except:
            raise ValueError("Failed to start chamber")
        # graceful end
        return True
    #*****************************


    #*****************************
    def stop(self):
        """
        Stops Temperature chamber

        Return
            False: Something went wrong
            True:  Successful
        """
        # stop chamber
        try:
            self.set_mode(sh641Const.MODE_STANDBY)  # bring to standby
            self.set_power(sh641Const.PWR_OFF)      # disable
        except:
            raise ValueError("Failed to stop chamber")
        # graceful end
        return True
    #*****************************
    
    
    #*****************************
    def info(self):
        """
        Returns Humidity and Temperature resolution in fracs
        """
        # build fracs dict
        fracs = {}
        fracs['temperature'] = 1   # temperature frac digits
        fracs['humidity'] = 1      # humidity frac digits
        # build final info structure
        info = {}
        info['fracs'] = fracs           # add
        info['name'] = "ESPEC_SH641"    # insert
        # release
        return info
    #***************************** 
        
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
if __name__ == '__main__':

    myChamber = especShSu()                         # call class constructor
    myChamber.open()                                # open with interface defaults
    print(myChamber.get_clima())                    # get current clima
    myChamber.start()                               # start chamber
    myChamber.set_clima(clima={'temperature': 25})  # set temperature value
    myChamber.stop()                                # stop chamber
    myChamber.close()                               # close handle
#------------------------------------------------------------------------------



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2020
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          ATWG.py
@date:          2020-01-04

@note           Arbitrary Temperature Waveform Generator
                  - directly started from command line
                  - run with 'python ./ATWG <myArgs>'
                  - 'run ./ATWG/ATWG.py --sine --chamber=SIM --minTemp=10 --maxTemp=60 --startTemp=30 --period=1h'

@see            https://docs.python.org/3/library/exceptions.html#exception-hierarchy
"""



#------------------------------------------------------------------------------
# Standard
import sys                          # python path handling
import os                           # platform independent paths
import argparse                     # argument parser
import time                         # time
import itertools                    # spinning progress bar
import re                           # regex, needed for number string separation
# Self
from ATWG.waves.waves import waves  # waveform generator
#------------------------------------------------------------------------------



#------------------------------------------------------------------------------
class ATWG:
    #*****************************
    def __init__(self):
        """
        Initialization
        """
        # config
        self.cfg_tsample_sec = 1                    # sample time is 1sec
        self.avlChambers = ["SIM", "ESPEC_SH641",]  # supported climate chambers
        # storing elements
        self.chamber = None     # class for chamber
        self.wave = None        # waveform
        self.clima = {}         # storage element for last measured clima
        # time string conversion
        self.timeToSec = {'s': 1, 'sec': 1, 'm': 60, 'min': 60, 'h': 3600, 'hour': 3600, 'd': 86400, 'day': 86400}   # conversion dictory to seconds
        self.timeColSep = "d:h:m:s"                                                                                  # colon separated time string prototype
        # Progress Spinner
        # SRC: https://stackoverflow.com/questions/4995733/how-to-create-a-spinning-command-line-cursor
        self.spinner = itertools.cycle(['-', '/', '|', '\\'])
    #*****************************
    
    
    #*****************************
    def parse_cli(self, cliArgs=None):
        """
        @note           Parses Arguments from Command line
        @see            https://docs.python.org/3.3/library/argparse.html
        
        @rtype          dict
        @return         parsed cli arguments to setup chamber and waveform
        """
        # create object
        parser = argparse.ArgumentParser(description="Arbitrary Temperature Waveform Generator")    # create class
        # waveform
        parser.add_argument('--sine',       action='store_true', help="sine waveform")                          # selects used waveform
        parser.add_argument('--trapezoid',  action='store_true', help="trapezoid waveform")                     #
        parser.add_argument('--invert',     action='store_true', help="wave starts with negative slew rate")    # w/o flag starts wave with positive slew, if set with negative slew
        # waveform parameters
        parser.add_argument("--period",    nargs=1, default=["1h",],    help="Period duration of selected waveform")            # interface
        parser.add_argument("--minTemp",   nargs=1, default=None,       help="waveforms minimal temperature value [C]")         # minimal temperature value
        parser.add_argument("--maxTemp",   nargs=1, default=None,       help="waveforms maximal temperature value [C]")         # maximal temperature value
        parser.add_argument("--startTemp", nargs=1, default=["25C",],   help="start temperature of waveform [C]")               # start temperature value
        parser.add_argument("--riseTime",  nargs=1, default=None,       help="change rate from lower to higher temperature")    # temperature change rate in postive temperature direction
        parser.add_argument("--fallTime",  nargs=1, default=None,       help="change rate from lower to higher temperature")    # temperature change rate in negative temperature direction
        # climate chamber
        parser.add_argument("--chamber",    nargs=1, default=[self.avlChambers[0], ],   help="Used climate chamber")                # selected temperature chamber, default ESPEC_SH641
        parser.add_argument("--itfCfgFile", nargs=1, default=None,                      help="Yml interface configuration file")    # interface
        # parse
        args = parser.parse_args(cliArgs)
        # select climate chamber
        chamberArgs = {}
        chamberArgs['chamber'] = args.chamber[0]        # chamber
        chamberArgs['itfCfgFile'] = args.itfCfgFile     # interface config
        # align CLI to wave.py api
        waveArgs = {}                                       # init dict
        waveArgs['ts'] = self.cfg_tsample_sec               # define sample time
        waveArgs['tp'] = self.time_to_sec(args.period[0])   # cast and align
        if ( args.sine and args.trapezoid ):                # dispatch waveform switch
            raise ValueError("Multiple waveform selected")
        elif ( args.sine ):             # sine selected
            waveArgs['wave'] = "sine"
        elif ( args.trapezoid ):        # trapszoid selected
            waveArgs['wave'] = "trapezoid"
        if ( None != args.minTemp ):    # align low temperature
            waveArgs['lowVal'] = float(args.minTemp[0].replace("C", "").replace("c", ""))
        if ( None != args.maxTemp ):    # align high temperature
             waveArgs['highVal'] = float(args.maxTemp[0].replace("C", "").replace("c", ""))
        if ( False == (('wave' in waveArgs) and ('lowVal' in waveArgs) and ('highVal' in waveArgs) ) ):  # check for mandatory args
            raise ValueError("Missing mandatory args: wave, lowVal, highVal")
        if ( None != args.startTemp ):
            waveArgs['initVal'] = float(args.startTemp[0].replace("C", "").replace("c", ""))
            waveArgs['initVal'] = max(min(waveArgs['initVal'], waveArgs['highVal']), waveArgs['lowVal']) # apply upper and lower fence
        if ( None != args.riseTime ):   # convert risetime
            waveArgs['tr'] = self.temp_grad_to_time(gradient=args.riseTime[0], deltaTemp=waveArgs['highVal']-waveArgs['lowVal'])
        if ( None != args.fallTime ):
            waveArgs['tf'] = self.temp_grad_to_time(gradient=args.fallTime[0], deltaTemp=waveArgs['highVal']-waveArgs['lowVal'])
        if ( args.invert ):
            waveArgs['pSlope'] = False
        # normal end
        return chamberArgs, waveArgs
    #*****************************

    
    #*****************************
    def time_to_sec(self, time=None):
        """
        @note           detects type of time and converts to seconds in numeric format
                        supported time formats
                          * d:h:m:s
                          * 1d, 1h, 1m, 1s
                            
        @param time     time in seconds or time string
        @rtype          float
        @return         time in seconds as numeric value
        """
        # check empty argument
        if ( None == time ):
            raise ValueError("No time string provided")
        # check if numeric data is provided, treated as seconds    
        elif ( isinstance(time, float) or isinstance(time, int) ):
            return time
        # check for string
        elif ( isinstance(time, str) ):
            # check for pure numeric string
            if ( time.isnumeric() ):
                return float(time)
            # check for ':' separated string
            elif ( -1 != time.find(":") ):
                # collects converted time
                secs = 0
                # prepare for positional to typ conv
                colSep = self.timeColSep.split(":") # make to list
                colSep.reverse()                    # alignment from seconds
                # iterate over segments in reversed order. last element is always sec
                for idx,item in enumerate(reversed(time.split(":"))):
                    # skip empty element
                    if ( 0 == len(item) ): continue
                    # convert to sec
                    try:
                        secs = secs + float(item) * self.timeToSec[colSep[idx]]     # accumulate and convert
                    except:
                        raise Warning("Skipping positional element " + str(idx) + " with value '" + str(item) + "'")
                # release result
                return secs
            # check if string contents elements from 'toSec'
            elif ( time.replace(" ", "").replace(".", "").isalnum() ):
                # prepare time
                secs = 0
                # separate into time substrings
                for timePart in time.split(" "):    # split at blank at iterate
                    # https://stackoverflow.com/questions/12409894/fast-way-to-split-alpha-and-numeric-chars-in-a-python-string/12411196
                    # https://stackoverflow.com/questions/4703390/how-to-extract-a-floating-number-from-a-string
                    digUnit = re.findall(r"[a-zA-Z_]+|[-+]?\d*\.\d+|\d+", timePart)   # split number from unit
                    if ( 1 == len(digUnit) ):       # handle time w/o numeric multiplier
                        digUnit.append(digUnit[0])  # save base time
                        digUnit[0] = 1              # complete non time base part
                    elif ( 2 < len(digUnit) ):
                        raise Warning("Time string part '" + timePart + "' not convertable, skip...")
                        continue
                    # convert to seconds
                    try:
                        secs = secs + float(digUnit[0]) * self.timeToSec[digUnit[1]]
                    except:
                        raise Warning ("Time unit '" + digUnit[1] + "' unknown")
                # release result
                return secs
            # unrecognized time string
            else:
                raise ValueError("Unrecognized time string '" + time + "'")
        # unkown data type
        else:
            raise TypeError("Unsupported data type '" + str(type(time)) + "'")
    #*****************************
    
    
    #*****************************
    def sec_to_time(self, sec=None, sep=" "):
        """
        @note           Converts given seconds to human readable time format
                        supported time formats
                          * d:h:m:s
                          * 1d, 1h, 1m, 1s
                          
        @param sec      time in seconds, number to convert
        @param sep      separator between time string { col | blank }
        
        @rtype          string
        @return         seconds converted to time string
        """
        # check empty argument
        if ( None == sec ):
            raise ValueError("No time provided")
        if ( 0 > sec ):
            raise ValueError("Only non-negativ secs allowed")
        if ( 0 == sec):
            return (str(0))
        # prepare data set
        secToTime = dict(zip(self.timeToSec.values(), self.timeToSec.keys()))   # swap keys and values
        secIter = sec;                                                          # store in intermediate variable
        timeStr = ""
        # extract base unit and split
        for unitDiv in list(reversed(sorted(secToTime.keys()))):
            # split base unit
            q, r = divmod(secIter, int(unitDiv))
            if (q > 0):
                # base not achieved
                if ( unitDiv > 1 ):
                    timeStr = timeStr + str(int(q))
                # base resolution achieved, take also fracs
                else:
                    timeStr = timeStr + str(secIter)
                # dispatch separator
                if ( sep == " " ):
                    timeStr = timeStr + secToTime[unitDiv][0] + " "
                elif( sep == ":" ):
                    timeStr = timeStr + ":"
                else:
                    raise ValueError("Unsupported separator '" + sep + "'")
                # update remaining time
                secIter = secIter - q*int(unitDiv)
        # drop last char 
        timeStr = timeStr[0:-1]
        # release result
        return timeStr
    #*****************************
    
    
    #*****************************
    def temp_grad_to_time(self, gradient=None, deltaTemp=None):    
        """
        @note               converts given gradient or slew time to seconds
                            supported formats:
                              * 5C/min
                              * 5C/5min
                              * 1min
                            
        @param gradient     time in seconds, number to convert
        @param deltaTemp    min/max temperature difference
        @rtype              float
        @return             slew time from min to max
        """
        # check for gradient
        if ( None == gradient ):
            raise ValueError("No temperature gradient given")
        # check for gradient
        if ( -1 != gradient.find("/") ):
            # check
            if ( None == deltaTemp ):
                raise ValueError("Low/High Temperature value for total slew time required")
            # split time and temp
            temperature = gradient.split("/")[0]
            timeVal = gradient.split("/")[1]
            # prepare
            timeVal = self.time_to_sec(time=timeVal)                        # convert to seconds
            temperature = temperature.replace("C", "").replace("c", "")     # filter celsius
            temperature = temperature.replace("K", "").replace("k", "")     # filter kelvin
            # calc slew time
            slewTime = (abs(deltaTemp) / float(temperature)) * timeVal
        # slew time over complete range
        else:
            slewTime = self.time_to_sec(time=gradient)
        # normal end
        return slewTime
    #*****************************
    
    
    #*****************************
    def normalize_gradient(self, grad_sec=None):
        """
        @note               normalizes dividend with increasing divisor with 
                            the goal to achieve a one in the dividend
                            
        @param grad_sec     time in seconds, number to convert
        @rtype              string
        @return             human readable gradient
        """
        # check for gradient
        if ( None == grad_sec ):
            raise ValueError("No temperature gradient given")
        # stuck rates smaller then 8 fracs to zero
        if ( 0 == float("{num:+.{frac}f}".format(num=grad_sec, frac=8))):
            return {'val': grad_sec, 'base': 's'}
        # determine timebase to once digit
        for tb in self.timeToSec.values():
            digit = float(grad_sec) * float(tb)
            base = tb
            if ( abs(digit) >= 1.0 ):
                break
        # build result dict
        try:        # conv to human readable time
            return {'val': digit, 'base': self.sec_to_time(sec=base)[1:]}
        except:     # no conv in case of ero
            return {'val': grad_sec, 'base': 's'}
    #*****************************
    
    
    #*****************************
    def open(self, chamberArg=None, waveArg=None):
        """
        @note               prepare chamber and opens for operation
                              * opens interface to chamber
                              * initializes waveform
                            
        @param chamberArg   climate chamber setting
        @param waveArg      waveform settings
        @rtype              boolean
        @return             successful
        """        
        # check for args
        if ( None == chamberArg or None == waveArg ):
            raise ValueError("Missing args")
        # select chamber
        if ( "sim" == chamberArg['chamber'].lower() ):
            from ATWG.driver.sim.simChamber import simChamber   # import if required
            self.chamber = simChamber()
        elif ( "espec_sh641" == chamberArg['chamber'].lower() ):
            from ATWG.driver.espec.sh641 import especShSu       # import if required
            self.chamber = especShSu()                          # init driver
        else:
            raise ValueError("Unsupported climate chmaber '" + chamberArg['chamber'] +"' selected")
        # open chamber interface
        if ( None != chamberArg['itfCfgFile'] ):
            self.chamber.open(cfgFile = chamberArg['itfCfgFile'])
        else:
            self.chamber.open()
        # init waveform
        self.wave = waves()         # create class
        self.wave.set(**waveArg)    # init waveform
        # normal end
        return True
    #*****************************
    
    
    #*****************************
    def start(self):
        """
        @note               starts chamber with operation
                              * set temperature is current temperature
                            
        @rtype              boolean
        @return             successful
        """
        # check for successfull opening
        if ( (None == self.chamber) or (None == self.wave) ):
            raise ValueError("Interfaces not opened, call methode 'open'")
        # set current clima as target clima
        self.chamber.set_clima(clima=self.chamber.get_clima())
        # start chamber
        self.chamber.start()
        # graceful end
        return True
    #*****************************
    
    
    #*****************************
    def stop(self):
        """
        @note               stops chamber operation and set temperature to waveform start value
                            
        @rtype              boolean
        @return             successful
        """
        # set chamber to start value
        self.chamber.set_clima(clima={'temperature': self.wave.waveArgs['initVal']})
        # stop chamber
        self.chamber.stop()
        # graceful end
        return True
    #*****************************
    
    
    #*****************************
    def chamber_update(self):
        """
        @note               updates chamber settings
                              * reads from chamber current clima conditions
                              * set new temperture values
                            
        @rtype              boolean
        @return             successful
        """
        # check for successfull opening
        if ( (None == self.chamber) or (None == self.wave) ):
            raise ValueError("Interfaces not opened, call methode 'open'")
        # acquire current clima
        self.clima['get'] = self.chamber.get_clima();
        # calc next clima value
        self.clima['set'] = self.wave.next();
        # set chamber value
        self.chamber.set_clima(clima={'temperature': self.clima['set']['val']})
        # graceful end
        return True
    #*****************************
    
    
    #*****************************
    def cli_update(self):
        """
        @note       Updates command line interface output, clears complete
                    comman line window output and rewrites it
                            
        @rtype      boolean
        @return     successful
        """
        # acquire vals
        numFracs = self.chamber.info()['fracs']['temperature']
        grad_norm = self.normalize_gradient(grad_sec=self.clima['set']['grad'])
        # Update CLI Interface
        print("\x1b[2J")       # delete complete output
        print("Arbitrary Temperature Waveform Generator")
        print()
        print("  Chamber")
        print("    State    : Run " + self.spinner.__next__())
        print("    Type     : " + self.chamber.info()['name'])
        print("    Tmeas    : " + "{num:+.{frac}f} °C".format(num=self.clima['get']['temperature'], frac=numFracs))
        print("    Tset     : " + "{num:+.{frac}f} °C".format(num=self.clima['set']['val'], frac=numFracs))
        print()
        print("  Waveform")
        print("    Shape    : " + self.wave.waveArgs['wave'])
        print("    Tmin     : " + "{num:+.{frac}f} °C".format(num=self.wave.waveArgs['lowVal'], frac=numFracs))
        print("    Tmax     : " + "{num:+.{frac}f} °C".format(num=self.wave.waveArgs['highVal'], frac=numFracs))
        print("    Period   : " + self.sec_to_time(sec=self.wave.waveArgs['tp']))
        print("    Gradient : " + "{num:+.{frac}f} °C".format(num=grad_norm['val'], frac=numFracs) + "/" + grad_norm['base'])
        print()
        print()
        print("Press 'CTRL + C' for exit")
        # graceful end 
        return True
    #*****************************
    
    
    #*****************************
    def close(self):
        """
        @note               cloeses hw handle to chamber
                            
        @rtype              boolean
        @return             successful
        """
        # close chamber handle
        self.chamber.close()
        # graceful end
        return True
    #*****************************
    
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
if __name__ == '__main__':
    
    myATWG = ATWG()                                                                                         # constuct class
    chamberArg, waveArg = myATWG.parse_cli(["--sine", "--minTemp=20C", "--maxTemp=30C", "--chamber=SIM"])   # set option
    myATWG.open(chamberArg=chamberArg, waveArg=waveArg)                                                     # init waveformgenertor and open chamber interface
    myATWG.chamber_update()                                                                                 # update chamber
    myATWG.cli_update()                                                                                     # update UI
    myATWG.stop()
    myATWG.close()
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2019
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          waves.py
@date:          2019-08-24

@note           calculates discrete waveforms
"""



#------------------------------------------------------------------------------
import math     # sine
#------------------------------------------------------------------------------



#------------------------------------------------------------------------------
class waves:
    """
    @note:  class to create numeric waveforms
    """

    #*****************************
    def __init__(self):
        """
        @note:          initializes class
        """
        self.waveDescr = {} # descriptor of initializes waveform
        self.iterator = 0   # waveform iterator
        self.waveArgs = {}  # not initialized
    #*****************************


    #*****************************
    def divide(self, dividend, divisor):
        """
        @note   calculates quotient and return nan in case of zero devision

        @param dividend     input number which should divided
        @param divisor      divides dividend

        @return             quotient of division
        """
        try:
            return float(dividend/divisor)
        except ZeroDivisionError:
            if ( 0 < dividend ):
                return float('inf')
            elif ( 0 > dividend ):
                return float('-inf')
            else:
                return float('nan')
    #*****************************


    #*****************************
    def set(self, **kwargs):
        """
        @note       selects waveform, and initializes waveform with proper arguments

        @see        sine()
        @see        trapezoid()

        @return:    True
        """
        # prepare
        self.waveDescr = {}     # reset wave descriptor
        self.waveArgs = {}      # make invalid
        waveParam = {}          # for waveform construction
        # assign kwargs to dict
        for key, value in kwargs.items():
            # separate waveform description from waveform selection
            if ( key != "wave" ):
                waveParam[key] = value
            # assign to storage element
            self.waveArgs[key] = value
        # init waveform
        if ( "sine" == self.waveArgs['wave'] ):
            (self.iterator, self.waveDescr) = self.sine(**waveParam)        # sine
        elif ( "trapezoid" == self.waveArgs['wave'] ):
            (self.iterator, self.waveDescr) = self.trapezoid(**waveParam)   # trapezoid
        else:
            raise ValueError("Unsupported waveform '" + self.waveArgs['wave'] + "' requested")
        # normal end
        return True
    #*****************************


    #*****************************
    def next(self):
        """
        @note       go one discrete time step forward

        @return     dict with new temperature and gradient
        """
        # init
        newVal = {}
        # in case of non intinilaized waveform is waveArgs not avialable
        try:
            # dispatch
            if ( "sine" == self.waveArgs['wave'] ):
                (self.iterator, newVal) = self.sine(descr=(self.iterator,self.waveDescr))       # update
            elif ( "trapezoid" == self.waveArgs['wave'] ):
                (self.iterator, newVal) = self.trapezoid(descr=(self.iterator,self.waveDescr))  # update
            else:
                raise ValueError("Unsupported waveform")
        except:
            raise ValueError("Uninitialized waveform")
        # return new vals
        return newVal
    #*****************************


    #*****************************
    def sine(self, descr=None, **kwargs):
        """
        @note           initializes and calculates next value

        @param descr    Sample/Update time of waveform  in seconds
        @param ts       Sample/Update time of waveform in base time units (f.e. seconds)
        @param tp       Period time of waveform in base time units (f.e. seconds)
        @param lowVal   minimal value
        @param highVal  maximal value
        @param initVal  initializes sine with value
        @param pSlope   selects positive/negative slope side of sine, only evaluated in init
        @return         wave descriptor or next value tupple
        @see            test_sine_init, test_sine for usage
        """
        # init phase
        if ( None == descr ):
            # optarg defaults
            optarg = {}
            optarg['ts'] = 1                    # sampling time
            optarg['tp'] = 3600                 # period time in sampling time steps
            optarg['lowVal'] = 0                # low value of sine
            optarg['highVal'] = 1               # high value of sine
            optarg['initVal'] = float("nan")    # init iterator for this start value
            optarg['pSlope'] = True             # waveform increases with time
            # check optional arguments
            for key, value in kwargs.items():
                if ( key in optarg ):
                    optarg[key] = value
                else:
                    raise ValueError("Unknown optional argument '" + key + "'")
            # define length of wave in discrete steps
            x = {}                              # waveforms time behaviour
            x['ts'] = optarg['ts']              # sample rate
            x['tp'] = optarg['tp']              # periode
            x['n'] = round(x['tp']/x['ts'])     # number of steps for full periode
            # value base of waveform
            y = {}
            y['amp'] = (optarg['highVal'] - optarg['lowVal']) / 2   # sine amplitude
            y['ofs'] = (optarg['highVal'] + optarg['lowVal']) / 2   # sine offset
            # find start value of iterator
            if ( math.isnan(optarg['initVal']) ):
                iterator = 0
            else:
                # ensure inside temp range
                initVal = min(optarg['initVal'], optarg['highVal'])   # apply upper fence
                initVal = max(initVal, optarg['lowVal'])              # applay lower fence
                # init wave iterator
                iterator = (x['tp']/(2*math.pi*x['ts'])) * math.asin((initVal-y['ofs'])/y['amp'])
                iterator = round(iterator)
                # shift to first period
                if ( iterator < 0 ):
                    iterator  += x['n']
                if ( iterator > x['n']-1 ):
                    iterator  -= x['n']
                # change slew direction
                if ( True == optarg['pSlope'] ):
                    if ( 0.25*x['n'] < iterator <= 0.5*x['n'] ):        # shift to [0*n, 0.25*n]
                        iterator = 0.25*x['n'] - iterator
                    elif ( 0.5*x['n'] < self.iterator <= 0.75*x['n'] ): # shift to [0.75*n, n]
                        iterator = 0.75*x['n'] + (0.75*x['n']-iterator)
                else:
                    if ( 0 <= iterator <= 0.25*x['n']):                 # shift to [0.25*n, 0.5*n]
                        iterator = 0.25*x['n'] + (0.25*x['n'] - iterator)
                    elif ( 0.75*x['n'] < self.iterator <= x['n']):      # shift to [0.5*n, 0.75*n]
                        iterator = 0.75*x['n'] - (iterator - 0.75*x['n'])
            # build waveform
            wave = {}
            wave['x'] = x
            wave['y'] = y
            # return iterator and wave
            return (iterator, wave)
        # calculate next temp value
        else:
            # disassemble descriptor
            iterator, wave = descr
            # calculate next time step
            new = {}
            new['val'] = wave['y']['ofs'] + wave['y']['amp']*(math.sin(2*math.pi*((iterator)*(float(1)/wave['x']['n']))))                       # calculate discrete sine value for n
            new['grad'] = wave['y']['amp']*(2*math.pi*(float(1)/wave['x']['n']))*(math.cos(2*math.pi*((iterator)*(float(1)/wave['x']['n']))))   # calc gradient, derived discrete sine
            # prepare for next calc
            iterator += 1
            # jump to sine start
            if ( iterator > wave['x']['n']-1 ):
                iterator -= wave['x']['n']
            # assign to release tupple
            return (iterator, new)
    #*****************************


    #*****************************
    def trapezoid(self, descr=None, **kwargs):
        """
        @note               generates discrete trapezoid waveform

        @param descr        waveform descriptor, generated by this function in init phase
        @param ts           Sample/Update time of waveform  in seconds
        @param tp           Period time of waveform in seconds
        @param lowVal       minimal value of trapezoid
        @param highVal      maximal value of trapezoid
        @param tr           rise time in sec from min to max
        @param tf           fall time in sec from max to min
        @param dutyCycle    ratio from max-to-min except the transition times (rise/fall)
        @param pSlope       selects positive/negativ slope side of sine, only evaluated in init
        @param initVal      initializes sine with value
        @return             wave descriptor or next value tupple
        @rtype              
        @see                test_trapezoid_init, test_trapezoid for usage
        """
        # init phase
        if ( None == descr ):
            # optarg defaults
            optarg = {}
            optarg['ts'] = 1                    # sampling time
            optarg['tp'] = 3600                 # period time in sampling time steps
            optarg['lowVal'] = 0                # low value of sine
            optarg['highVal'] = 1               # high value of sine
            optarg['pSlope'] = True             # waveform increases with vals
            optarg['tf'] = 0                    # fall time, time from highVal to lowVal
            optarg['tr'] = 0                    # rise time
            optarg['initVal'] = float("nan")    # init iterator for this start value
            optarg['dutyCycle'] = float("nan")  # ratio high/low val time
            # check optional arguments
            for key, value in kwargs.items():
                if ( key in optarg ):
                    optarg[key] = value
                else:
                    raise ValueError("Unknown optional argument '" + key + "'")
            # w/ rise/fall time does wave exist
            if ( 0 > (optarg['tp'] - optarg['tf'] - optarg['tr']) ):
                raise ValueError("Rise + Fall time is larger then period, minimal period length is " + str(optarg['tf']+optarg['tr']) +"s")
            # define length of wave in discrete steps
            x = {}                                      # waveforms time behavior
            x['ts'] = optarg['ts']                      # sample rate
            x['tp'] = optarg['tp']                      # period
            x['n'] = round(optarg['tp']/optarg['ts'])   # number of steps for full period
            # determine discrete time steps of waveform
            # duty cycle is given
            if ( False == math.isnan(optarg['dutyCycle']) ):
                # calculate discrete time steps
                step_rise_n = round(optarg['tr']/x['ts'])                                   # number of steps for rise
                step_fall_n = round(optarg['tf']/x['ts'])                                   # number of steps for fall
                step_high_n = round((x['n']-step_rise_n-step_fall_n)*optarg['dutyCycle'])   # number of steps for high
                step_low_n = x['n']-step_rise_n-step_fall_n-step_high_n                     # number of steps for low
            # next wave style

            # not unambiguously described
            else:
                raise ValueError("Provided arguments does not unambiguously describe the waveform")
            # build piecewise continuous function
            # {'min'}: iter_min, {'max'}: iter_max, {'grad'}: gradient, {'start'}: start value for this segment
            y = {}
            y['rise'] = {'start': 0,                   'stop': step_rise_n-1,                     'grad': self.divide(optarg['highVal']-optarg['lowVal'], step_rise_n), 'val': optarg['lowVal']}     # rise part
            y['high'] = {'start': y['rise']['stop']+1, 'stop': y['rise']['stop']+1+step_high_n-1, 'grad': 0,                                                            'val': optarg['highVal']}    # high part
            y['fall'] = {'start': y['high']['stop']+1, 'stop': y['high']['stop']+1+step_fall_n-1, 'grad': self.divide(optarg['lowVal']-optarg['highVal'], step_fall_n), 'val': optarg['highVal']}    # fall part
            y['low']  = {'start': y['fall']['stop']+1, 'stop': y['fall']['stop']+1+step_low_n-1,  'grad': 0,                                                            'val': optarg['lowVal']}     # low part
            # find start value of iterator
            if ( math.isnan(optarg['initVal']) ):
                iterator = 0
            else:
                # ensure inside temp range
                initVal = min(optarg['initVal'], optarg['highVal'])     # apply upper fence
                initVal = max(initVal, optarg['lowVal'])                # apply lower fence
                # init wave iterator
                if ( (True == optarg['pSlope']) and (0 != step_rise_n) ):       # waveform rises smooth
                    iterator = y['rise']['start'] + round((initVal-optarg['lowVal'])/(abs(optarg['highVal']-optarg['lowVal'])/step_rise_n))
                elif ( (False == optarg['pSlope']) and (0 != step_fall_n) ):    # waveform falls smooth
                    iterator = y['fall']['start'] + round((optarg['highVal']-initVal)/(abs(optarg['highVal']-optarg['lowVal'])/step_fall_n))
                else:                                                           # brick wall
                    iterator = 0
            # build final and return
            wave = {}
            wave['x'] = x
            wave['y'] = y
            return (iterator, wave)
        # calculate next step
        else:
            # disassemble descriptor
            iterator, wave = descr
            # calc waveform
            new = {}
            for foo, y in wave['y'].items():
                # match part of waveform
                if ( y['start'] <= iterator <= y['stop'] ):
                    new['val'] = y['val'] + y['grad'] * (iterator-y['start'])   # new value value
                    new['grad'] = y['grad'] / wave['x']['ts']                   # gradient per sec
            # inc wave iterator, prepare for next calc
            iterator += 1
            # jump to start
            if ( iterator > wave['x']['n']-1 ):
                iterator -= wave['x']['n']
            # assign to release tupple
            return (iterator, new)
    #*****************************

#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
if __name__ == '__main__':

    # create object
    myWave = waves()
    # check trapezoid function
    myWave.trapezoid_init(sample=1, period=3600, low=-10, high=30, rise=10, fall=20, init=23, posSlope=True)
    print(str(myWave.iterator))
    print(myWave.trapezoid())
    # check sine function
    myWave.sine_init(sample=1, period=3600, low=-10, high=30, init=23, posSlope=True)
    print(str(myWave.iterator))
    print(myWave.sine())
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
"""
@author:        Andreas Kaeberlein
@copyright:     Copyright 2021
@credits:       AKAE

@license:       GPLv3
@maintainer:    Andreas Kaeberlein
@email:         andreas.kaeberlein@web.de

@file:          setup.py
@date:          2021-01-02

@note           setup pip installer
"""



#------------------------------------------------------------------------------
# Python Libs
#
import setuptools
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name='ATWG',
    version='0.1.4',
    scripts=['atwg-cli'],
    install_requires = ["pyserial", "pyyaml"],
    author='Andreas Kaeberlein',
    author_email="andreas.kaeberlein@web.de",
    license="GPLv3",
    platforms="Any",
    description="Various Temperature Waveform Generator for Climate Chambers",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/akaeba/ATWG",
    download_url = "https://pypi.org/project/ATWG/",
    packages=["ATWG",
              "ATWG.waves",
              "ATWG.driver",
              "ATWG.driver.espec",
              "ATWG.driver.sim",
              ],                                        # define package to add
    package_data={"ATWG": ["driver/espec/*.yml"],},     # adds .yml config files to package
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)
#------------------------------------------------------------------------------
